package be.cylab.java.wowa.training;


import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.StreamHandler;

/**
 * @author CYLAB
 */
public final class Example {

    private Example() {

    }

    /**
     * Run algorithm with webshell dataset.
     *
     * @param args
     */
    public static void main(final String[] args) {
        int population_size = Integer.parseInt(args[0]);
        int crossover_rate = Integer.parseInt(args[1]);
        int mutation_rate = Integer.parseInt(args[2]);
        int max_generation_number = Integer.parseInt(args[3]);
        int selection_method;
        if (args[4].matches("RWS")) {
            selection_method = TrainerParameters.SELECTION_METHOD_RWS;
        } else if (args[4].matches("TOS")) {
            selection_method = TrainerParameters.SELECTION_METHOD_TOS;
        } else {
            throw new IllegalArgumentException(
                    "Selection method must be RWS ar TOS");
        }
        int generation_population_method;
        if (args[5].matches("RANDOM")) {
            generation_population_method
                    = TrainerParameters.POPULATION_INITIALIZATION_RANDOM;
        } else if (args[5].matches("QUASI_RANDOM")) {
            generation_population_method
                    = TrainerParameters.POPULATION_INITIALIZATION_QUASI_RANDOM;
        } else {
            throw new IllegalArgumentException(
                    "Initialization must be RANDOM or QUASI_RANDOM");
        }

        String data_file = "./ressources/webshell_data.json";
        String expected_file = "./ressources/webshell_expected.json";
        Logger logger = Logger.getLogger(Trainer.class.getName());
        logger.setLevel(Level.INFO);
        StreamHandler handler = new StreamHandler(System.out,
                new SimpleFormatter());
        logger.addHandler(handler);
        TrainerParameters parameters = new TrainerParameters(
                logger, population_size,
                crossover_rate,
                mutation_rate,
                max_generation_number,
                selection_method,
                generation_population_method);
        Trainer trainer = new Trainer(parameters, new SolutionDistance(5));

        long start_time = System.currentTimeMillis();
        //AbstractSolution solution = trainer.run(
        //        "./ressources/webshell_data.json",
        //        "./ressources/webshell_expected.json");
        //System.out.println(solution);
        long end_time = System.currentTimeMillis();
        logger.log(Level.INFO, "Execution time : "
                + (end_time - start_time) / 1000 + " seconds");

        System.out.println("Run Cross validation");
        List<AbstractSolution> solutions = trainer.runKFold(
                data_file,
                expected_file,
                10);

        for (int i = 0; i < solutions.size(); i++) {
            System.out.println(solutions.get(i));
            System.out.println(solutions.get(i).computeAUC(
                    data_file,
                    expected_file));
        }

    }

}
